@@ -7,7 +7,7 @@
  * Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *    $Header: /usr/local/devel/pglite/cvs/src/backend/bootstrap/bootstrap.c,v 1.20 1996/02/24 00:03:17 jolly Exp $
+ *    $Header: /usr/local/cvsroot/postgres95/src/backend/bootstrap/bootstrap.c,v 1.4 1996/07/23 02:23:05 scrappy Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -136,6 +136,9 @@ static char *relname;                   /* current relation name */
 AttributeTupleForm attrtypes[MAXATTR];  /* points to attribute info */
 static char	*values[MAXATTR];	/* cooresponding attribute values */
 int		numattr;		/* number of attributes for cur. rel */
+#ifdef OPENLINK_PATCHES
+extern int    fsyncOff;                 /* do not fsync the database */
+#endif
 
 #if defined(WIN32) || defined(PORTNAME_next)
 static jmp_buf    Warn_restart;
@@ -184,7 +187,9 @@ typedef void (*sig_func)();
  *	error handling / abort routines
  * ----------------
  */
-#if !defined(PORTNAME_bsdi)
+# if !defined(PORTNAME_BSD44_derived) && \
+     !defined(PORTNAME_bsdi) && \
+     !defined(PORTNAME_bsdi_2_1)
 void err()
 {
     Warnings++;
@@ -198,9 +203,16 @@ void err()
 static void
 usage()
 {
+#ifdef OPENLINK_PATCHES
+    fprintf(stderr,"Usage: postgres -boot [-d] [-C] [-F] [-O] [-Q] [-P portno] [dbName]\n");
+#else
     fprintf(stderr,"Usage: postgres -boot [-d] [-C] [-O] [-Q] [-P portno] [dbName]\n");
+#endif
     fprintf(stderr,"     d: debug mode\n");
     fprintf(stderr,"     C: disable version checking\n");
+#ifdef OPENLINK_PATCHES
+    fprintf(stderr,"     F: turn off fsync\n");
+#endif
     fprintf(stderr,"     O: set BootstrapProcessing mode\n");
     fprintf(stderr,"     P portno: specify port number\n");
 
@@ -257,7 +269,11 @@ BootstrapMain(int argc, char *argv[])
     Noversion = 0;
     dbName = NULL;
     
+#ifdef OPENLINK_PATCHES
+    while ((flag = getopt(argc, argv, "dCOQP:F")) != EOF) {
+#else
     while ((flag = getopt(argc, argv, "dCOQP")) != EOF) {
+#endif
 	switch (flag) {
 	case 'd':
 	    DebugMode = 1; /* print out debuggin info while parsing */
@@ -265,6 +281,11 @@ BootstrapMain(int argc, char *argv[])
 	case 'C':
 	    Noversion = 1; 
 	    break;
+#ifdef OPENLINK_PATCHES
+        case 'F':
+            fsyncOff = 1;
+            break;
+#endif
 	case 'O':
 	    override = true;
 	    break;
