@@ -6,20 +6,13 @@
  *
  * Copyright (c) 1994, Regents of the University of California
  *
- * $Id: execdefs.h,v 1.3 1995/02/12 09:12:13 andrew Exp $
+ * $Id: execdefs.h,v 1.2 1996/07/30 07:45:29 scrappy Exp $
  *
  *-------------------------------------------------------------------------
  */
 #ifndef EXECDEFS_H
 #define EXECDEFS_H
 
-/* ----------------
- *	executor scan direction definitions
- * ----------------
- */
-#define EXEC_FRWD		1		/* Scan forward */
-#define EXEC_BKWD		-1		/* Scan backward */
-
 /* ----------------
  *	ExecutePlan() tuplecount definitions
  * ----------------
