@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *    $Header: /usr/local/devel/pglite/cvs/src/backend/tcop/postgres.c,v 1.34 1996/02/24 00:47:04 jolly Exp $
+ *    $Header: /usr/local/cvsroot/postgres95/src/backend/tcop/postgres.c,v 1.4 1996/07/22 23:00:26 scrappy Exp $
  *
  * NOTES
  *    this is the "main" module of the postgres backend and
@@ -93,6 +93,10 @@ CommandDest whereToSendOutput;
 extern int	lockingOff;
 extern int	NBuffers;
 
+#ifdef OPENLINK_PATCHES
+int	fsyncOff = 0;
+#endif
+
 int	dontExecute = 0;
 static int	ShowStats;
 static bool	IsEmptyQuery = false;
@@ -107,6 +111,7 @@ jmp_buf    Warn_restart;
 #else
 sigjmp_buf Warn_restart;
 #endif /*defined(WIN32) || defined(PORTNAME_next) */
+int InWarn;
 
 extern int	NBuffers;
 
@@ -696,15 +701,22 @@ her exceeded legal ranges or was a divide by zero");
 
 static void usage(char* progname)
 {
+#ifdef OPENLINK_PATCHES
+    fprintf(stderr,"\t[-P portno] [-t tracetype] [-x opttype] [-bCEiLFNopQSs] [dbname]\n");
+#else
     fprintf(stderr, 
 	    "Usage: %s [-B nbufs] [-d lvl] ] [-f plantype] \t[-m portno] [\t -o filename]\n",
 	    progname);
     fprintf(stderr,"\t[-P portno] [-t tracetype] [-x opttype] [-bCEiLNopQSs] [dbname]\n");
+#endif
     fprintf(stderr, "    b: consider bushy plan trees during optimization\n");
     fprintf(stderr, "    B: set number of buffers in buffer pool\n");
     fprintf(stderr, "    C: supress version info\n");
     fprintf(stderr, "    d: set debug level\n");
     fprintf(stderr, "    E: echo query before execution\n");
+#ifdef OPENLINK_PATCHES
+    fprintf(stderr, "    F: turn off fsync\n");
+#endif
     fprintf(stderr, "    f: forbid plantype generation\n");
     fprintf(stderr, "    i: don't execute the query, just show the plan tree\n");
     fprintf(stderr, "    L: turn off locking\n");
@@ -804,7 +816,11 @@ PostgresMain(int argc, char *argv[])
 	hostName = hostbuf;
     }
 
+#ifdef OPENLINK_PATCHES
+    while ((flag = getopt(argc, argv, "B:bCd:Ef:iLm:MNo:P:pQSst:x:F")) != EOF)
+#else
     while ((flag = getopt(argc, argv, "B:bCd:Ef:iLm:MNo:P:pQSst:x:")) != EOF)
+#endif
 	switch (flag) {
 	    
 	case 'b':
@@ -851,6 +867,16 @@ PostgresMain(int argc, char *argv[])
 	    flagE = 1;
 	    break;
 	    
+#ifdef OPENLINK_PATCHES
+        case 'F':
+            /* --------------------
+             *  turn off fsync
+             * --------------------
+             */
+            fsyncOff = 1;
+            break;
+#endif
+
 	case 'f':
 	    /* -----------------
 	     *    f - forbid generation of certain plans
@@ -1178,6 +1204,7 @@ PostgresMain(int argc, char *argv[])
 #else
     if (setjmp(Warn_restart) != 0) {
 #endif /* WIN32 */
+	InWarn = 1;
 
 	time(&tim);
 	
@@ -1188,6 +1215,7 @@ PostgresMain(int argc, char *argv[])
 	
 	AbortCurrentTransaction();
     }
+    InWarn = 0;
     
     /* ----------------
      *	POSTGRES main processing loop begins here
@@ -1195,7 +1223,7 @@ PostgresMain(int argc, char *argv[])
      */
     if (IsUnderPostmaster == false) {
 	puts("\nPOSTGRES backend interactive interface");
-	puts("$Revision: 1.34 $ $Date: 1996/02/24 00:47:04 $");
+	puts("$Revision: 1.4 $ $Date: 1996/07/22 23:00:26 $");
     }
     
     /* ----------------
