@@ -7,11 +7,12 @@
  *
  *
  * IDENTIFICATION
- *    $Header: /usr/local/devel/pglite/cvs/src/libpq/fe-exec.c,v 1.10 1996/02/24 01:22:23 jolly Exp $
+ *    $Header: /usr/local/cvsroot/postgres95/src/libpq/fe-exec.c,v 1.12 1996/07/31 18:40:09 scrappy Exp $
  *
  *-------------------------------------------------------------------------
  */
 #include <stdlib.h>
+#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -19,6 +20,17 @@
 #include "libpq/pqcomm.h"
 #include "libpq-fe.h"
 #include <signal.h>
+#include <sys/ioctl.h>
+#include <sys/termios.h>
+
+#ifdef TIOCGWINSZ
+struct winsize screen_size;
+#else
+struct winsize {
+  int ws_row;
+  int ws_col;
+} screen_size;
+#endif
 
 /* the tuples array in a PGresGroup  has to grow to accommodate the tuples */
 /* returned.  Each time, we grow by this much: */
@@ -358,12 +370,20 @@ PQexec(PGconn* conn, char* query)
   char cmdStatus[MAX_MESSAGE_LEN];
   char pname[MAX_MESSAGE_LEN]; /* portal name */
   PGnotify *newNotify;
-  FILE *Pfin = conn->Pfin;
-  FILE *Pfout = conn->Pfout;
-  FILE* Pfdebug = conn->Pfdebug;
+  FILE *Pfin, *Pfout, *Pfdebug;
 
   pname[0]='\0';
 
+  if (!conn) return NULL;
+  if (!query) {
+    sprintf(conn->errorMessage, "PQexec() -- query pointer is null.");
+    return NULL;
+  }
+
+  Pfin = conn->Pfin;
+  Pfout = conn->Pfout;
+  Pfdebug = conn->Pfdebug;
+
   /*clear the error string */
   conn->errorMessage[0] = '\0';
 
@@ -500,6 +520,9 @@ PGnotify*
 PQnotifies(PGconn *conn)
 {
     Dlelem *e;
+
+    if (!conn) return NULL;
+
     if (conn->status != CONNECTION_OK) 
 	return NULL;
     /* RemHead returns NULL if list is empy */
@@ -531,6 +554,8 @@ int
 PQgetline(PGconn *conn, char *s, int maxlen)
 {
     int c = '\0';
+
+    if (!conn) return EOF;
     
     if (!conn->Pfin || !s || maxlen <= 1)
 	return(EOF);
@@ -561,7 +586,7 @@ PQgetline(PGconn *conn, char *s, int maxlen)
 void
 PQputline(PGconn *conn, char *s)
 {
-    if (conn->Pfout) {
+    if (conn && (conn->Pfout)) {
 	(void) fputs(s, conn->Pfout);
 	fflush(conn->Pfout);
     }
@@ -580,8 +605,12 @@ int
 PQendcopy(PGconn *conn)
 {
     char id;
-    FILE *Pfin = conn->Pfin;
-    FILE* Pfdebug = conn->Pfdebug;
+    FILE *Pfin, *Pfdebug;
+
+    if (!conn) return (int)NULL;
+
+    Pfin = conn->Pfin;
+    Pfdebug = conn->Pfdebug;
 
     if ( (id = pqGetc(Pfin,Pfdebug)) > 0)
 	return(0);
@@ -626,10 +655,7 @@ fill (int length, int max, char filler, FILE *fp)
 
 /*
  * PQdisplayTuples()
- *
- * a better version of PQprintTuples()
- * that can optionally do padding of fields with spaces and use different
- * field separators 
+ * kept for backward compatibility
  */
 void
 PQdisplayTuples(PGresult *res,
@@ -642,35 +668,21 @@ PQdisplayTuples(PGresult *res,
 {
 #define DEFAULT_FIELD_SEP " "
 
-    char *pager;
     int i, j;
     int nFields;
     int nTuples;
     int fLength[MAX_FIELDS];
-    int usePipe = 0;
 
     if (fieldSep == NULL)
 	fieldSep == DEFAULT_FIELD_SEP;
 
-    if (fp == NULL) 
-	fp = stdout;
-    if (fp == stdout) {
-	/* try to pipe to the pager program if possible */
-	pager=getenv("PAGER");
-	if (pager != NULL) {
-	    fp = popen(pager, "w");
-	    if (fp) {
-		usePipe = 1;
-		signal(SIGPIPE, SIG_IGN);
-	    } else
-		fp = stdout;
-	}
-    }
-
     /* Get some useful info about the results */
     nFields = PQnfields(res);
     nTuples = PQntuples(res);
   
+    if (fp == NULL) 
+	fp = stdout;
+
     /* Zero the initial field lengths */
     for (j=0  ; j < nFields; j++) {
       fLength[j] = strlen(PQfname(res,j));
@@ -685,7 +697,7 @@ PQdisplayTuples(PGresult *res,
 	    }
 	}
     }
-
+    
     if (printHeader) {
 	/* first, print out the attribute names */
 	for (i=0; i < nFields; i++) {
@@ -721,10 +733,6 @@ PQdisplayTuples(PGresult *res,
 		 (PQntuples(res) == 1) ? "" : "s");
   
     fflush(fp);
-    if (usePipe) {
-	pclose(fp);
-	signal(SIGPIPE, SIG_DFL);
-    }
 }
 
 
@@ -732,11 +740,8 @@ PQdisplayTuples(PGresult *res,
 /*
  * PQprintTuples()
  *
- * This is the routine that prints out the tuples that
- *  are returned from the backend.
- * Right now all columns are of fixed length,
- * this should be changed to allow wrap around for
- * tuples values that are wider.
+ * kept for backward compatibility
+ *
  */
 void
 PQprintTuples(PGresult *res,
@@ -804,6 +809,362 @@ PQprintTuples(PGresult *res,
     }
 }
 
+/*
+ * PQprint()
+ *
+ * new PQprintTuples routine (proff@suburbia.net)
+ * PQprintOpt is a typedef (structure) that containes
+ * various flags and options. consult libpq-fe.h for
+ * details
+ */
+
+void
+PQprint(FILE *fout,
+              PGresult *res,
+	      PQprintOpt *po
+              )
+{
+    int nFields;
+
+    nFields = PQnfields(res);
+
+    if ( nFields > 0 ) {  /* only print tuples with at least 1 field.  */
+	int i,j;
+	int nTups;
+	int *fieldMax=NULL;	/* in case we don't use them */
+	unsigned char *fieldNotNum=NULL;
+	char *border=NULL;
+	char **fields=NULL;
+	char **fieldNames;
+	int fieldMaxLen=0;
+	int numFieldName;
+	int fs_len=strlen(po->fieldSep);
+	int total_line_length = 0;
+	int usePipe = 0;
+	char *pagerenv;
+	char buf[8192*2+1];
+
+	nTups = PQntuples(res);
+	if (!(fieldNames=(char **)calloc(nFields, sizeof (char *))))
+	{
+		perror("calloc");
+		exit(1);
+	}
+	if (!(fieldNotNum=(unsigned char *)calloc(nFields, 1)))
+	{
+		perror("calloc");
+		exit(1);
+	}
+	if (!(fieldMax=(int *)calloc(nFields, sizeof(int))))
+	{
+		perror("calloc");
+		exit(1);
+	}
+	for (numFieldName=0; po->fieldName && po->fieldName[numFieldName]; numFieldName++)
+		;
+	for (j=0; j < nFields; j++)
+	{
+		int len;
+		char *s=(j<numFieldName && po->fieldName[j][0])? po->fieldName[j]: PQfname(res, j);
+		fieldNames[j]=s;
+		len=s? strlen(s): 0;
+		fieldMax[j] = len;
+		/*
+		if (po->header && len<5)
+			len=5; 
+	        */
+		len+=fs_len;
+		if (len>fieldMaxLen)
+			fieldMaxLen=len;
+		total_line_length += len;
+	}
+ 
+	total_line_length += nFields * strlen(po->fieldSep) + 1;
+ 
+	if (fout == NULL) 
+		fout = stdout;
+	if (po->pager && fout == stdout &&
+		isatty(fileno(stdin)) &&
+		isatty(fileno(stdout)))
+	{
+		/* try to pipe to the pager program if possible */
+#ifdef TIOCGWINSZ
+		if (ioctl(fileno(stdout),TIOCGWINSZ,&screen_size) == -1 ||
+		    screen_size.ws_col == 0 ||
+		    screen_size.ws_row == 0)
+		{
+#endif
+			screen_size.ws_row = 24;
+			screen_size.ws_col = 80;
+#ifdef TIOCGWINSZ
+		}
+#endif
+		pagerenv=getenv("PAGER");
+		if (pagerenv != NULL &&
+		    pagerenv[0] != '\0' && 
+		   !po->html3 &&
+		   ((po->expanded &&
+			nTups * (nFields+1) >= screen_size.ws_row) ||
+		    (!po->expanded &&
+			nTups * (total_line_length / screen_size.ws_col + 1) *
+				( 1 + (po->standard != 0)) >=
+			screen_size.ws_row -
+			(po->header != 0) *
+				(total_line_length / screen_size.ws_col + 1) * 2
+ 			- (po->header != 0) *2 /* row count and newline */
+			)))
+		{
+			fout = popen(pagerenv, "w");
+			if (fout) {
+				usePipe = 1;
+				signal(SIGPIPE, SIG_IGN);
+			} else
+				fout = stdout;
+		}
+	}
+ 
+	if (!po->expanded && (po->align || po->html3))
+	{
+		if (!(fields=(char **)calloc(nFields*(nTups+1), sizeof(char *))))
+		{
+			perror("calloc");
+			exit(1);
+		}
+	}
+	else
+		if (po->header && !po->html3)
+        	{
+			if (po->expanded)
+			{
+				if (po->align)
+					fprintf(fout, "%-*s%s Value\n", fieldMaxLen-fs_len, "Field", po->fieldSep);
+				else
+					fprintf(fout, "%s%sValue\n", "Field", po->fieldSep);
+			}
+			else
+			{
+				int len=0;
+				for (j=0; j < nFields; j++)
+				{
+					char *s=fieldNames[j];
+					fputs(s, fout);
+					len+=strlen(s)+fs_len;
+					if ((j+1)<nFields)
+						fputs(po->fieldSep, fout);
+				}
+				fputc('\n', fout);
+				for (len-=fs_len; len--; fputc('-', fout));
+				fputc('\n', fout);
+			}
+		}
+	if (po->expanded && po->html3)
+	{
+		if (po->caption)
+			fprintf(fout, "<centre><h2>%s</h2></centre>\n", po->caption);
+		else
+			fprintf(fout, "<centre><h2>Query retrieved %d tuples * %d fields</h2></centre>\n", nTups, nFields);
+	}
+        for (i = 0; i < nTups; i++)
+	{
+	    if (po->expanded)
+	    {
+	    	if (po->html3)
+			fprintf(fout, "<table %s><caption align=high>%d</caption>\n", po->tableOpt? po->tableOpt: "", i);
+		else
+			fprintf(fout, "-- RECORD %d --\n", i);
+	    }
+            for (j = 0; j < nFields; j++)
+	    {
+                char *pval, *p, *o;
+		int plen;
+		if ((plen=PQgetlength(res,i,j))<1 || !(pval=PQgetvalue(res,i,j)) || !*pval)
+		{
+			if (po->align || po->expanded)
+				continue;
+			goto efield;
+		}
+		for (p=pval, o=buf; *p; *(o++)=*(p++))
+		{
+			if ((fs_len==1 && (*p==*(po->fieldSep))) || *p=='\\')
+				*(o++)='\\';
+			if (po->align && !((*p >='0' && *p<='9') || *p=='.' || *p=='E' || *p=='e' || *p==' ' || *p=='-'))
+				fieldNotNum[j]=1;
+		}
+		*o='\0';
+		if (!po->expanded && (po->align || po->html3))
+		{
+			int n=strlen(buf);
+			if (n>fieldMax[j])
+				fieldMax[j]=n;
+			if (!(fields[i*nFields+j]=(char *)malloc(n+1)))
+			{
+				perror("malloc");
+				exit(1);
+			}
+			strcpy(fields[i*nFields+j], buf);
+		}
+		else
+		{
+			if (po->expanded)
+			{
+				if (po->html3)
+					fprintf(fout, "<tr><td align=left><b>%s</b></td><td align=%s>%s</td></tr>\n",
+						fieldNames[j], fieldNotNum[j]? "left": "right", buf);
+				else
+				{
+					if (po->align)
+						fprintf(fout, "%-*s%s %s\n", fieldMaxLen-fs_len, fieldNames[j], po->fieldSep, buf);
+					else
+						fprintf(fout, "%s%s%s\n", fieldNames[j], po->fieldSep, buf);
+				}
+			}
+			else
+			{
+				if (!po->html3)
+				{
+					fputs(buf, fout);
+efield:
+					if ((j+1)<nFields)
+						fputs(po->fieldSep, fout);
+					else
+						fputc('\n', fout);
+				}
+			}
+		}
+	    }
+	    if (po->html3 && po->expanded)
+	    	fputs("</table>\n", fout);
+    	}
+	if (!po->expanded && (po->align || po->html3))
+	{
+	    	if (po->html3)
+		{
+			if (po->header)
+			{
+				if (po->caption)
+			                fprintf(fout, "<table %s><caption align=high>%s</caption>\n", po->tableOpt? po->tableOpt: "", po->caption);
+				else
+					fprintf(fout, "<table %s><caption align=high>Retrieved %d tuples * %d fields</caption>\n", po->tableOpt? po->tableOpt: "", nTups, nFields);
+			} else
+			        fprintf(fout, "<table %s>", po->tableOpt? po->tableOpt: "");
+		}
+		if (po->header)
+        	{
+			if (po->html3)
+				fputs("<tr>", fout);
+			else
+			{
+				int tot=0;
+				int n=0;
+				char *p;
+				for (; n<nFields; n++)
+					tot+=fieldMax[n]+fs_len+(po->standard? 2: 0);
+				if (po->standard)
+					tot+=fs_len*2+2;
+				if (!(p=border=malloc(tot+1)))
+				{
+					perror("malloc");
+					exit(1);
+				}
+				if (po->standard)
+				{
+					char *fs=po->fieldSep;
+					while (*fs++)
+						*p++='+';
+				}
+				for (j=0; j <nFields; j++)
+				{
+					int len;
+					for (len=fieldMax[j] + (po->standard? 2:0) ; len--; *p++='-');
+					if (po->standard || (j+1)<nFields)
+					{
+						char *fs=po->fieldSep;
+						while (*fs++)
+							*p++='+';
+					} 
+				}
+				*p='\0';
+				if (po->standard)
+					fprintf(fout, "%s\n", border);
+			}
+			if (po->standard)
+				fputs(po->fieldSep, fout);
+                	for (j=0; j < nFields; j++)
+			{
+				char *s=PQfname(res, j);
+				if (po->html3)
+				{
+					fprintf(fout, "<th align=%s>%s</th>", fieldNotNum[j]? "left": "right",
+						fieldNames[j]);
+				}
+				else
+				{
+					int n=strlen(s);
+					if (n>fieldMax[j])
+						fieldMax[j]=n;
+					if (po->standard)
+						fprintf(fout, fieldNotNum[j]? " %-*s ": " %*s ", fieldMax[j], s);
+					else
+						fprintf(fout, fieldNotNum[j]? "%-*s": "%*s", fieldMax[j], s);
+					if (po->standard || (j+1)<nFields)
+						fputs(po->fieldSep, fout);
+				}
+			}
+			if (po->html3)
+				fputs("</tr>\n", fout);
+			else
+				fprintf(fout, "\n%s\n", border);
+		}
+        	for (i = 0; i < nTups; i++)
+		{
+			if (po->html3)
+				fputs("<tr>", fout);
+			else
+				if (po->standard)
+					fputs(po->fieldSep, fout);
+			
+           		for (j = 0; j < nFields; j++)
+			{
+				char *p=fields[i*nFields+j];
+			 	if (po->html3)
+			 		fprintf(fout, "<td align=%s>%s</td>", fieldNotNum[j]? "left": "right", p? p: "");
+
+				else
+				{
+			 		fprintf(fout, fieldNotNum[j]? (po->standard? " %-*s ": "%-*s"): (po->standard? " %*s ": "%*s"), fieldMax[j], p? p: "");
+					if (po->standard || (j+1)<nFields)
+						fputs(po->fieldSep, fout);
+				}
+				if (p)
+					free(p);
+			}
+			if (po->html3)
+				fputs("</tr>", fout);
+			else
+				if (po->standard)
+					fprintf(fout, "\n%s", border);
+			fputc('\n', fout);
+		}
+		free(fields);
+	}
+	if (po->header && !po->html3)
+		fprintf (fout, "(%d row%s)\n\n",PQntuples(res),
+			 (PQntuples(res) == 1) ? "" : "s");
+	free(fieldMax);
+	free(fieldNotNum);
+	free(fieldNames);
+	if (usePipe)
+	{
+		pclose(fout);
+		signal(SIGPIPE, SIG_DFL);
+	}
+	if (border)
+		free(border);
+	if (po->html3 && !po->expanded)
+		fputs("</table>\n", fout);
+	}
+}
+
 
 /* ----------------
  *	PQfn -  Send a function call to the POSTGRES backend.
@@ -836,12 +1197,16 @@ PQfn(PGconn *conn,
      PQArgBlock *args,
      int nargs)
 {
-    FILE *Pfin = conn->Pfin;
-    FILE *Pfout = conn->Pfout;
-    FILE* Pfdebug = conn->Pfdebug;
+    FILE *Pfin, *Pfout, *Pfdebug;
     int id;
     int i;
 
+    if (!conn) return NULL;
+
+    Pfin = conn->Pfin;
+    Pfout = conn->Pfout;
+    Pfdebug = conn->Pfdebug;
+
     /* clear the error string */
     conn->errorMessage[0] = '\0';
 
@@ -908,26 +1273,36 @@ PQfn(PGconn *conn,
     }
 }
 
-
-
-
 /* ====== accessor funcs for PGresult ======== */
 
 ExecStatusType 
 PQresultStatus(PGresult* res)
 { 
+    if (!res) {
+      fprintf(stderr, "PQresultStatus() -- pointer to PQresult is null");
+      return PGRES_NONFATAL_ERROR;
+    }
+
     return res->resultStatus; 
 }
 
 int 
 PQntuples(PGresult *res) 
 {
+    if (!res) {
+      fprintf(stderr, "PQntuples() -- pointer to PQresult is null");
+      return (int)NULL;
+    }
     return res->ntups;
 }
 
 int
 PQnfields(PGresult *res) 
 {
+    if (!res) {
+      fprintf(stderr, "PQnfields() -- pointer to PQresult is null");
+      return (int)NULL;
+    }
     return res->numAttributes;
 }
 
@@ -937,6 +1312,11 @@ PQnfields(PGresult *res)
 char* 
 PQfname(PGresult *res, int field_num) 
 {
+    if (!res) {
+      fprintf(stderr, "PQfname() -- pointer to PQresult is null");
+      return NULL;
+    }
+
     if (field_num > (res->numAttributes - 1))  {
 	fprintf(stderr,
 		"PQfname: ERROR! name of field %d(of %d) is not available", 
@@ -957,6 +1337,11 @@ PQfnumber(PGresult *res, char* field_name)
 {
   int i;
 
+  if (!res) {
+    fprintf(stderr, "PQfnumber() -- pointer to PQresult is null");
+    return -1;
+  }
+
   if (field_name == NULL ||
       field_name[0] == '\0' ||
       res->attDescs == NULL)
@@ -973,6 +1358,11 @@ PQfnumber(PGresult *res, char* field_name)
 Oid
 PQftype(PGresult *res, int field_num) 
 {
+    if (!res) {
+      fprintf(stderr, "PQftype() -- pointer to PQresult is null");
+      return InvalidOid;
+    }
+
     if (field_num > (res->numAttributes - 1))  {
 	fprintf(stderr,
 		"PQftype: ERROR! type of field %d(of %d) is not available", 
@@ -987,6 +1377,11 @@ PQftype(PGresult *res, int field_num)
 int2
 PQfsize(PGresult *res, int field_num) 
 {
+    if (!res) {
+      fprintf(stderr, "PQfsize() -- pointer to PQresult is null");
+      return (int2)NULL;
+    }
+
     if (field_num > (res->numAttributes - 1))  {
 	fprintf(stderr,
 		"PQfsize: ERROR! size of field %d(of %d) is not available", 
@@ -999,6 +1394,10 @@ PQfsize(PGresult *res, int field_num)
 }
 
 char* PQcmdStatus(PGresult *res) {
+  if (!res) {
+    fprintf(stderr, "PQcmdStatus() -- pointer to PQresult is null");
+    return NULL;
+  }
   return res->cmdStatus;
 }
 
@@ -1008,6 +1407,11 @@ char* PQcmdStatus(PGresult *res) {
     if not, return ""
 */
 char* PQoidStatus(PGresult *res) {
+  if (!res) {
+    fprintf(stderr, "PQoidStatus() -- pointer to PQresult is null");
+    return NULL;
+  }
+
   if (!res->cmdStatus)
     return "";
 
@@ -1031,13 +1435,17 @@ char* PQoidStatus(PGresult *res) {
 char* 
 PQgetvalue(PGresult *res, int tup_num, int field_num)
 {
+    if (!res) {
+      fprintf(stderr, "PQgetvalue() -- pointer to PQresult is null");
+      return NULL;
+    }
+
     if (tup_num > (res->ntups - 1) ||
 	field_num > (res->numAttributes - 1))  {
 	fprintf(stderr,
 		"PQgetvalue: ERROR! field %d(of %d) of tuple %d(of %d) is not available", 
 		field_num, res->numAttributes - 1, tup_num, res->ntups);
     }
-	
     
     return res->tuples[tup_num][field_num].value;
 }
@@ -1050,6 +1458,11 @@ PQgetvalue(PGresult *res, int tup_num, int field_num)
 int
 PQgetlength(PGresult *res, int tup_num, int field_num)
 {
+    if (!res) {
+      fprintf(stderr, "PQgetlength() -- pointer to PQresult is null");
+        return (int)NULL;
+    }
+
     if (tup_num > (res->ntups - 1 )||
 	field_num > (res->numAttributes - 1))  {
 	fprintf(stderr,
